Attribute VB_Name = "Module2"
' Region aus Grafik erstellen
' Copyright  2012 by Zardoz

Option Explicit

' Typen
' Bitmap Header Definition
Private Type BITMAP
  bmType As Long
  bmWidth As Long
  bmHeight As Long
  bmWidthBytes As Long
  bmPlanes As Integer
  bmBitsPixel As Integer
  bmBits As Long
End Type

' structures for defining DIBs
Private Type BITMAPINFOHEADER
  biSize As Long
  biWidth As Long
  biHeight As Long
  biPlanes As Integer
  biBitCount As Integer
  biCompression As Long
  biSizeImage As Long
  biXPelsPerMeter As Long
  biYPelsPerMeter As Long
  biClrUsed As Long
  biClrImportant As Long
End Type

Private Type RGBQUAD
  rgbBlue As Byte
  rgbGreen As Byte
  rgbRed As Byte
  rgbReserved As Byte
End Type

Private Type BITMAPINFO
  bmiHeader As BITMAPINFOHEADER
  bmiColors As RGBQUAD
End Type

' Deklarationen
' gdi32
Private Declare Function GetObjectAPI Lib "gdi32" Alias "GetObjectA" (ByVal hObject As Long, ByVal nCount As Long, lpObject As Any) As Long
Private Declare Function ExtCreateRegion Lib "gdi32" (lpXform As Any, ByVal nCount As Long, lpRgnData As Any) As Long
Private Declare Function GetDIBits Lib "gdi32" (ByVal aHDC As Long, ByVal hBitmap As Long, ByVal nStartScan As Long, ByVal nNumScans As Long, lpBits As Any, lpBI As BITMAPINFO, ByVal wUsage As Long) As Long
Public Declare Function CreateCompatibleDC Lib "gdi32" (ByVal hdc As Long) As Long
Public Declare Function DeleteDC Lib "gdi32" (ByVal hdc As Long) As Long
Public Declare Function SelectObject Lib "gdi32" (ByVal hdc As Long, ByVal hObject As Long) As Long

' Konstanten
' DIB color table identifiers
Private Const DIB_RGB_COLORS As Long = 0 ' color table in RGBs

' constants for the biCompression field
Private Const BI_RGB As Long = 0

' Region-Type
Private Const RDH_RECTANGLES As Long = 1
'

Public Function CreateRegionFromPicture(PicHandle As Long, _
  Optional Mask As Long = -1) As Long
  ' Region aus Bitmap generieren
  
  Dim SW As Long, SH As Long, x1 As Long, y1 As Long
  Dim Z1 As Long, P1hdc As Long, P1OldHandle As Long
  Dim MapData() As Long, Buffer() As Long
  Dim MapInfo As BITMAPINFO
  
  If PicHandle = 0 Then
    MsgBox "Error:" & vbCr & "Handle = 0", vbExclamation + vbOKOnly
    CreateRegionFromPicture = 0
    Exit Function
  End If
  
  Call GetMapMetrics(PicHandle, SW, SH)
  ReDim MapData(SW - 1, SH - 1), Buffer(8 + 4& * SW * SH - 1)
  
  With MapInfo.bmiHeader
    .biSize = Len(MapInfo.bmiHeader)
    .biWidth = SW
    .biHeight = -SH
    .biPlanes = 1
    .biBitCount = 32
    .biCompression = BI_RGB
    .biSizeImage = 0
    .biXPelsPerMeter = 0
    .biYPelsPerMeter = 0
    .biClrUsed = 0
    .biClrImportant = 0
  End With
  
  With MapInfo.bmiColors
    .rgbBlue = 0
    .rgbGreen = 0
    .rgbRed = 0
    .rgbReserved = 0
  End With

  ' Bilddaten in Array einlesen
  P1hdc = CreateCompatibleDC(0)
  P1OldHandle = SelectObject(P1hdc, PicHandle)
  Call GetDIBits(P1hdc, PicHandle, 0, SH, _
    MapData(0, 0), MapInfo, DIB_RGB_COLORS)
  Call SelectObject(P1hdc, P1OldHandle)
  Call DeleteDC(P1hdc)
  
  If Mask = -1 Then
    ' Maskenfarbe holen. Bytes sind bereits getauscht
    Mask = MapData(0, 0)
  Else
    ' Bytes Maskenfarbe tauschen
    Mask = (Mask And vbRed) * &H10000 + (Mask And vbGreen) + _
      (Mask And vbBlue) \ &H10000
  End If
  
  Z1 = 8
  For y1 = 0 To SH - 1
    For x1 = 0 To SW - 1
      If MapData(x1, y1) <> Mask Then
        Buffer(Z1) = x1
        Buffer(Z1 + 1) = y1
        Buffer(Z1 + 2) = x1 + 1
        Buffer(Z1 + 3) = y1 + 1
        Z1 = Z1 + 4
      End If
    Next x1
  Next y1
  Buffer(0) = 32 ' Headerlnge
  Buffer(1) = RDH_RECTANGLES ' Regiontyp
  Buffer(2) = Z1 / 4 - 2 ' Anzahl Rechtecke
  
  ' Region erzeugen
  CreateRegionFromPicture = _
    ExtCreateRegion(ByVal 0&, 4& * Z1, Buffer(0))
  
  ' Speicher freigeben
  Erase MapData, Buffer

End Function

Public Sub GetMapMetrics(PicHandle As Long, _
  SW As Long, SH As Long)
  ' Breite und Hhe einer Bitmap
  
  Dim Bmp As BITMAP
  
  Call GetObjectAPI(PicHandle, Len(Bmp), Bmp)
  SW = Bmp.bmWidth
  SH = Bmp.bmHeight

End Sub
